<?xml version="1.0" encoding="UTF-8"?>

<!--*********************************************************************************************************** 
	created by: Laura Robinson
	contact: lrobinson@secstate.wa.gov
	create date: 2009-05-01
	notes: this file was written and tested with the SAXON 6.5.3 transformation processor and is meant to 
	tranform the input file (e.g. reelTbl.xml) to recreate the appropriate database table while comparing it 
	to the output from the batch files (batch.xml and mets reel files). It recreates the table with color-coded 
	backgrounds indicating possible missing or mismatched data. 

	to do:
	1. need to better target StartDate and EndDate
	2. need to create a loop for density readings
	3. research dvv, see if we need to check more than just the first mapped occurance
	4. may need to change cataloging practices - check for org codes and labels - why are we cataloging w/ commas?
	***********************************************************************************************************-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:mods="http://www.loc.gov/mods/v3" xmlns:ndnp="http://www.loc.gov/ndnp" xmlns:mets="http://www.loc.gov/METS/" version="1.1">
	<xsl:output method="html"/>
	<!--*******************************************************************************************************
		param: passed in from command line via the run_report.bat file
		*******************************************************************************************************-->	
	<xsl:param name="batchFilePath"/>
	<!--*******************************************************************************************************
		declare keys: used to match test output for missing data from reelTbl
		*******************************************************************************************************-->
	<!--batchReels key used to make sure batch includes the reel record first and also used to create list of unique reel #s-->	
	<xsl:key name="batchReels" match="ndnp:batch/ndnp:reel" use="@reelNumber"/>
	<!--*****************************************************
		NOTE: keys below double as field names for reelTbl 
		WARNING: don't alter or add to @name attributes
		*****************************************************-->	
	<!--ndnp:titles is not all reel titles - only the directory title - not a true test for multiple title reels-->
	<xsl:key name="Titles" match="//ndnp:titles" use="."/>
	<!--lccn not represented in mets reel - directory name checked instead during titleTbl test-->
	<xsl:key name="lccn" match="lccn" use="."/>
	<xsl:key name="ReelNumber" match="//mods:identifier[@type='reel number']" use="."/>
	<!--ndnp:startDate is really first date of title (not reel) - not a true test for multiple title reels-->
	<xsl:key name="StartDate" match="//ndnp:startDate" use="."/>
	<!--ndnp:endDate is really last date of title (not reel) - not a true test for multiple title reels-->
	<xsl:key name="EndDate" match="//ndnp:endDate" use="."/>
	<xsl:key name="Position" match="//ndnp:position" use="."/>
	<xsl:key name="ReductionRatio" match="//ndnp:reductionRatio" use="."/>
	<xsl:key name="CaptureResolutionOriginal" match="//ndnp:captureResolutionOriginal" use="."/>
	<xsl:key name="CaptureResolutionFilm" match="//ndnp:captureResolutionFilm" use="."/>
	<xsl:key name="GuideToContentsPresent_x0020_flag" match="//ndnp:guideToContentsPresentFlag" use="."/>
	<xsl:key name="GuideToContentsString" match="//ndnp:guideToContentsString" use="."/>
	<xsl:key name="DateMicrofilmCreated" match="//ndnp:dateMicrofilmCreated" use="."/>
	<xsl:key name="LooseLeavesFlag" match="//ndnp:looseLeavesFlag" use="."/>
	<xsl:key name="BoundVolumeFlag" match="//ndnp:boundVolumeFlag" use="."/>
	<xsl:key name="Comments" match="//ndnp:comments" use="."/>
	<xsl:key name="Dimensions" match="//ndnp:dimensions" use="."/>
	<xsl:key name="PagesPerIssue" match="//ndnp:pagesPerIssue" use="."/>
	<xsl:key name="NumberResolutionTargets" match="//ndnp:numberOfResolutionTargets" use="."/>
	<xsl:key name="ResolutionOfMaster" match="//ndnp:resolutionOfMaster" use="number(.)"/>
	<xsl:key name="ResolutionCommentMaster" match="//ndnp:resolutionCommentMaster" use="."/>
	<xsl:key name="DensityReadingMaster" match="//ndnp:densityReadingMaster" use="number(.)"/>
	<xsl:key name="AverageDensityMaster" match="//ndnp:averageDensityMaster" use="number(.)"/>
	<xsl:key name="DminMaster" match="//ndnp:dminMaster" use="number(.)"/>
	<xsl:key name="ResolutionOfDuplicateNegative" match="//ndnp:resolutionOfDuplicateNegative" use="number(.)"/>
	<xsl:key name="ResolutionCommentDuplicateNegative" match="//ndnp:resolutionCommentDuplicateNegative" use="."/>
	<xsl:key name="DensityReadingDuplicateNegative" match="//ndnp:densityReadingDuplicateNegative" use="."/>
	<xsl:key name="AverageDensityDuplicateNegative" match="//ndnp:averageDensityDuplicateNegative" use="number(.)"/>
	<xsl:key name="DminDuplicateNegative" match="//ndnp:dminDuplicateNegative" use="number(.)"/>
	<xsl:key name="DigitalResponsibleInstitution" match="//mods:note[@type='agencyResponsibleForReproduction']" use="@displayLabel"/>
	<xsl:key name="DigitalResponsibleInstitutionCode" match="//mods:note[@type='agencyResponsibleForReproduction']" use="."/>
	<xsl:key name="SourceRepository" match="//mods:physicalLocation" use="@displayLabel"/>
	<xsl:key name="SourceRepositoryCode" match="//mods:physicalLocation" use="."/>
	<xsl:key name="NoteToProcessor" match="NoteToProcessor" use="."/>
	<!--*******************************************************************************************************
		declare global variables: used to gain access to other documents or set constants throughout xsl file
		*******************************************************************************************************-->
	<xsl:variable name="batchFile" select="document($batchFilePath)"/>
	<xsl:variable name="thisFile" select="document('reelTbl.xsl')"/>
	<xsl:template match="dataroot">
		<!--*******************************************************************************************************
		test page header and body
		*******************************************************************************************************-->
		<html>
			<body>
				<h1>Test: reelTbl</h1>
				<!-- to gain access to root element attributes use * rather than . -->
				<h3>Batch: <xsl:value-of select="$batchFile/*/@name"/></h3>
				<!-- Ordered list of unique reels in batch.xml file -->
				<h3>List of reels in batch:</h3>
				<ol>
					<xsl:for-each select="$batchFile/ndnp:batch/ndnp:reel[generate-id(.) = generate-id(key('batchReels', @reelNumber)[1])]">
						<li>
							<xsl:value-of select="@reelNumber"/>
						</li>
					</xsl:for-each>
				</ol>
				<xsl:call-template name="table"/>
					<xsl:element name="a">
						<xsl:attribute name="href">titleTbl_report.html</xsl:attribute>
						<xsl:text>titleTbl_report.html</xsl:text>
					</xsl:element><br/>
					<xsl:element name="a">
						<xsl:attribute name="href">issueTbl_report.html</xsl:attribute>
						<xsl:text>issueTbl_report.html</xsl:text>
					</xsl:element>					
			</body>
		</html>
	</xsl:template>
	<!--*******************************************************************************************************
		table header and header row
		*******************************************************************************************************-->
	<xsl:template name="table">
		<h3>reelTbl:</h3>
				<xsl:text>Note: Titles in red are missing from </xsl:text>
				<a>
					<xsl:attribute name="href">
						<xsl:value-of select="$batchFilePath"/>
					</xsl:attribute>
					<xsl:attribute name="target">
						<xsl:text>_blank</xsl:text>
					</xsl:attribute>					
					<xsl:value-of select="$batchFilePath"/>
				</a>
		<br/>
		<xsl:text>Note: Cells in yellow may not be wrong but require further investigation.</xsl:text>
		<table border="1">
			<!--table header-->
			<tr>
				<!--navigates within this xsl file to the variables below to read all possible names and create field header-->
				<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchReels']">
					<td>
						<xsl:value-of select="@name"/>
					</td>
				</xsl:for-each>
			</tr>
			<xsl:call-template name="rows"/>
		</table>
	</xsl:template>
	<!--*******************************************************************************************************
		rows recreated from all reelTbl.xml reelTbl elements (each reelTbl is a reel record)
		*******************************************************************************************************-->
	<xsl:template name="rows">
		<xsl:for-each select="reelTbl">
			<!--select the whole reelTbl record-->
			<xsl:variable name="reelTblRec" select="."/>
			<!--tests to see if there is a batch.xml @reelNumber match-->
			<xsl:variable name="reelMatch">
				<!--must first get into the context of Batch.xml in order to test the value -->
				<xsl:for-each select="$batchFile">
					<!--key() function returns the matching node. Therefore, apply-templates checks for value-->
					<xsl:apply-templates select="key('batchReels', $reelTblRec/ReelNumber)"/>
				</xsl:for-each>
			</xsl:variable>
			<tr>
				<!--test to see if reel exists in batch.xml, if not turns the whole row red-->
				<xsl:choose>
					<xsl:when test="$reelMatch = ''">
						<xsl:call-template name="bgColorTest">
							<xsl:with-param name="color" select="'red'"/>
						</xsl:call-template>
						<!--build cells within row here using reelTbl records that don't have matching reel mets files-->
						<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchReels']">
							<xsl:variable name="fieldName" select="@name"/>
							<td>
								<xsl:value-of select="$reelTblRec/*[name() = $fieldName]"/>
							</td>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="cellMatch">
							<xsl:with-param name="reelTblRec" select="$reelTblRec"/>
							<xsl:with-param name="metsReelPath" select="$reelMatch"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</tr>
		</xsl:for-each>
	</xsl:template>
	<!--*******************************************************************************************************
		cell values recreated from reelTbl.xml reelTbl/* elements
		*******************************************************************************************************-->
	<xsl:template name="cellMatch">
		<xsl:param name="reelTblRec"/>
		<xsl:param name="metsReelPath"/>
		<!--automatically create a cell for each possible field name again by navigating to variables above and then input what comes from that variable-->
		<xsl:for-each select="$thisFile/xsl:stylesheet/xsl:key[@name != 'batchReels']">
			<xsl:variable name="fieldName" select="@name"/>
			<xsl:variable name="fieldMatch" select="@match"/>
			<td>
				<xsl:choose>
					<xsl:when test="$fieldName[. != 'lccn'][. != 'NoteToProcessor'] and $reelTblRec/*[name() = $fieldName] != ''">
						<xsl:call-template name="metsReelMatch">
							<xsl:with-param name="docPath" select="$metsReelPath"/>
							<xsl:with-param name="keyName" select="$fieldName"/>
							<xsl:with-param name="keyMatch" select="$fieldMatch"/>
							<xsl:with-param name="cellValue" select="$reelTblRec/*[name() = $fieldName]"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$reelTblRec/*[name() = $fieldName]"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</xsl:for-each>
	</xsl:template>
	<!--*******************************************************************************************************
		cell test using paths from Batch.xml to mets reel files
		*******************************************************************************************************-->
	<xsl:template name="metsReelMatch">
		<xsl:param name="docPath"/>
		<xsl:param name="keyName"/>
		<xsl:param name="keyMatch"/>
		<xsl:param name="cellValue"/>
		<xsl:param name="paramLoop"/>
		<!--need to be recursive so we have to test for and take string before '.xml' then add it back in-->
		<xsl:variable name="pathParse" select="concat(substring-before($batchFilePath, 'BATCH.xml'), substring-before(substring-after($docPath, './'), '.xml'), '.xml')"/>
		<xsl:variable name="metsFile" select="document($pathParse)"/>
		<xsl:variable name="leftOvers" select="substring-after($docPath, '.xml')"/>
		<xsl:variable name="metsMatch">
			<xsl:for-each select="$metsFile">
				<!--***********************************************************************************************
					WARNING: key() only checks the first occurance - need to investigate for-each loop possiblities 
					***********************************************************************************************-->
				<xsl:choose>
					<xsl:when test="$keyName = 'Titles'">
						<xsl:if test="not(contains($cellValue, key($keyName, $cellValue)))">
							<xsl:text>mismatch</xsl:text>
						</xsl:if>
					</xsl:when>
					<!--testing from BATCH.xml for all but first mets file on reel -->
					<xsl:when test="$keyName = 'StartDate' and $paramLoop != ''">
						<!--position of issue is first in title -->
						<xsl:variable name="batchDate" select="$batchFile/ndnp:batch/ndnp:issue[@lccn = substring-before(substring-after($docPath, './'), '/')]/@issueDate"/>
						<xsl:if test="not(key($keyName, $batchDate))">
							<xsl:text>Date from BATCH.xml: </xsl:text>
							<xsl:value-of select="$batchDate"/>
						</xsl:if>									
					</xsl:when>
					<!--testing from BATCH.xml for all but last mets file on reel -->					
					<xsl:when test="$keyName = 'EndDate' and $leftOvers != ''">
						<!--position of issue is last in title-->
						<xsl:variable name="batchDate" select="$batchFile/ndnp:batch/ndnp:issue[@lccn = substring-before(substring-after($docPath, './'), '/')][position() = last()]/@issueDate"/>
						<xsl:if test="not(key($keyName, $batchDate))">
							<xsl:text>Date from BATCH.xml: </xsl:text>
							<xsl:value-of select="$batchDate"/>
						</xsl:if>									
					</xsl:when>
					<xsl:when test="contains($keyName, 'DensityReading')">
						<xsl:variable name="readings">
							<xsl:for-each select="//*[name() = substring-after($keyMatch, '//')]">
								<xsl:value-of select="."/>
								<xsl:if test="position() != last()">
									<xsl:text>, </xsl:text>
								</xsl:if>	
							</xsl:for-each>
						</xsl:variable>
						<xsl:if test="normalize-space($cellValue) != $readings">
							<xsl:text>One or more readings mismatched: </xsl:text>
							<xsl:value-of select="$cellValue"/>
						</xsl:if>									
					</xsl:when>											
					<xsl:when test="$keyName = 'GuideToContentsPresent_x0020_flag' or $keyName = 'LooseLeavesFlag' or $keyName = 'BoundVolumeFlag'">
						<xsl:choose>
							<xsl:when test="$cellValue = '1'">
								<xsl:if test="not(key($keyName, 'true'))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:when>
							<xsl:when test="$cellValue = '0'">
								<xsl:if test="not(key($keyName, 'false'))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:when>
							<xsl:otherwise>
								<xsl:if test="not(key($keyName, $cellValue))">
									<xsl:text>mismatch</xsl:text>
								</xsl:if>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:when test="not(key($keyName, $cellValue))">
						<xsl:value-of select="'mismatch'"/>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
		</xsl:variable>
		<xsl:choose>
			<!--when there is an error, there will be a value in $metsMatch-->
			<xsl:when test="$metsMatch = 'mismatch'">
				<xsl:call-template name="bgColorTest">
					<xsl:with-param name="color" select="'yellow'"/>
				</xsl:call-template>
				<xsl:value-of select="$cellValue"/>
				<br/>
				<xsl:call-template name="metsLink">
					<xsl:with-param name="link" select="$pathParse"/>
					<xsl:with-param name="elementPath" select="$keyMatch"/> 
				</xsl:call-template>
			</xsl:when>
			<!--when there is a special error, a special value will be in $metsMatch-->
			<xsl:when test="$metsMatch != ''">
				<xsl:call-template name="bgColorTest">
					<xsl:with-param name="color" select="'yellow'"/>
				</xsl:call-template>
				<xsl:value-of select="$metsMatch"/>
				<br/>			
				<xsl:call-template name="metsLink">
					<xsl:with-param name="link" select="$pathParse"/>
					<xsl:with-param name="elementPath" select="$keyMatch"/> 
				</xsl:call-template>				
			</xsl:when>
			<!--when there is no error but more mets-reel documents call template again with $leftOvers-->
			<xsl:when test="$metsMatch = '' and $leftOvers != ''">
				<xsl:call-template name="metsReelMatch">
					<xsl:with-param name="docPath" select="$leftOvers"/>
					<xsl:with-param name="keyName" select="$keyName"/>
					<xsl:with-param name="keyMatch" select="$keyMatch"/>
					<xsl:with-param name="cellValue" select="$cellValue"/>
					<xsl:with-param name="paramLoop" select="$paramLoop + 1"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$cellValue"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!--*******************************************************************************************************
		sets the link to the cellValue
		*******************************************************************************************************-->
	<xsl:template name="metsLink">
		<xsl:param name="link"/>
		<xsl:param name="elementPath"/>
			<xsl:text>[mismatch in: </xsl:text>	
				<xsl:element name="a">
					<xsl:attribute name="href">
						<xsl:value-of select="$link"/>
					</xsl:attribute>
					<xsl:attribute name="target">
						<xsl:text>_blank</xsl:text>
					</xsl:attribute>
					<xsl:value-of select="$elementPath"/>
				</xsl:element>
			<xsl:text>]</xsl:text>	
	</xsl:template>	
	<!--*******************************************************************************************************
		sets the background color based on rules (match, mismatched, or missing)
		*******************************************************************************************************-->
	<xsl:template name="bgColorTest">
		<xsl:param name="color"/>
		<xsl:attribute name="bgcolor">
			<xsl:value-of select="$color"/>
		</xsl:attribute>
	</xsl:template>
</xsl:stylesheet>